/* siman/mgsl_siman.h
 * 
 * Copyright (C) 1996, 1997, 1998, 1999, 2000 Mark Galassi
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __MGSL_SIMAN_H__
#define __MGSL_SIMAN_H__
#include <stdlib.h>
#include <gsl/gsl_rng.h>

#undef __BEGIN_DECLS
#undef __END_DECLS
#ifdef __cplusplus
# define __BEGIN_DECLS extern "C" {
# define __END_DECLS }
#else
# define __BEGIN_DECLS /* empty */
# define __END_DECLS /* empty */
#endif

__BEGIN_DECLS

/* types for the function pointers passed to mgsl_siman_solve */

typedef double (*mgsl_siman_Efunc_t) (void *xp);
typedef void (*mgsl_siman_step_t) (const gsl_rng *r, void *xp, double step_size);
typedef double (*mgsl_siman_metric_t) (void *xp, void *yp);
typedef void (*mgsl_siman_print_t) (void *xp);
typedef void (*mgsl_siman_copy_t) (void *source, void *dest);
typedef void * (*mgsl_siman_copy_construct_t) (void *xp);
typedef void (*mgsl_siman_destroy_t) (void *xp);

/* this structure contains all the information needed to structure the
   search, beyond the energy function, the step function and the
   initial guess. */

typedef struct {
  int n_tries;          /* how many points to try for each step */
  int iters_fixed_T;    /* how many iterations at each temperature? */
  double ss_initial;    /* initial max step size in the random walk */
  double mu_ss;         /* How does ss decrement? i.e., */
  /* the following parameters are for the Boltzmann distribution */
  double k, t_initial, mu_t, t_min;
} mgsl_siman_params_t;

/* prototype for the workhorse function */

void mgsl_siman_solve(const gsl_rng * r, 
                     void *x0_p, mgsl_siman_Efunc_t Ef,
                     mgsl_siman_step_t take_step,
                     mgsl_siman_metric_t distance,
                     mgsl_siman_print_t print_position,
                     mgsl_siman_copy_t copyfunc,
                     mgsl_siman_copy_construct_t copy_constructor,
                     mgsl_siman_destroy_t destructor,
                     size_t element_size,
                     mgsl_siman_params_t params);

__END_DECLS

#endif /* __MGSL_SIMAN_H__ */
